
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_SPARC_NONE,         0)
ELF_RELOC(R_SPARC_8,            1)
ELF_RELOC(R_SPARC_16,           2)
ELF_RELOC(R_SPARC_32,           3)
ELF_RELOC(R_SPARC_DISP8,        4)
ELF_RELOC(R_SPARC_DISP16,       5)
ELF_RELOC(R_SPARC_DISP32,       6)
ELF_RELOC(R_SPARC_WDISP30,      7)
ELF_RELOC(R_SPARC_WDISP22,      8)
ELF_RELOC(R_SPARC_HI22,         9)
ELF_RELOC(R_SPARC_22,           10)
ELF_RELOC(R_SPARC_13,           11)
ELF_RELOC(R_SPARC_LO10,         12)
ELF_RELOC(R_SPARC_GOT10,        13)
ELF_RELOC(R_SPARC_GOT13,        14)
ELF_RELOC(R_SPARC_GOT22,        15)
ELF_RELOC(R_SPARC_PC10,         16)
ELF_RELOC(R_SPARC_PC22,         17)
ELF_RELOC(R_SPARC_WPLT30,       18)
ELF_RELOC(R_SPARC_COPY,         19)
ELF_RELOC(R_SPARC_GLOB_DAT,     20)
ELF_RELOC(R_SPARC_JMP_SLOT,     21)
ELF_RELOC(R_SPARC_RELATIVE,     22)
ELF_RELOC(R_SPARC_UA32,         23)
ELF_RELOC(R_SPARC_PLT32,        24)
ELF_RELOC(R_SPARC_HIPLT22,      25)
ELF_RELOC(R_SPARC_LOPLT10,      26)
ELF_RELOC(R_SPARC_PCPLT32,      27)
ELF_RELOC(R_SPARC_PCPLT22,      28)
ELF_RELOC(R_SPARC_PCPLT10,      29)
ELF_RELOC(R_SPARC_10,           30)
ELF_RELOC(R_SPARC_11,           31)
ELF_RELOC(R_SPARC_64,           32)
ELF_RELOC(R_SPARC_OLO10,        33)
ELF_RELOC(R_SPARC_HH22,         34)
ELF_RELOC(R_SPARC_HM10,         35)
ELF_RELOC(R_SPARC_LM22,         36)
ELF_RELOC(R_SPARC_PC_HH22,      37)
ELF_RELOC(R_SPARC_PC_HM10,      38)
ELF_RELOC(R_SPARC_PC_LM22,      39)
ELF_RELOC(R_SPARC_WDISP16,      40)
ELF_RELOC(R_SPARC_WDISP19,      41)
ELF_RELOC(R_SPARC_7,            43)
ELF_RELOC(R_SPARC_5,            44)
ELF_RELOC(R_SPARC_6,            45)
ELF_RELOC(R_SPARC_DISP64,       46)
ELF_RELOC(R_SPARC_PLT64,        47)
ELF_RELOC(R_SPARC_HIX22,        48)
ELF_RELOC(R_SPARC_LOX10,        49)
ELF_RELOC(R_SPARC_H44,          50)
ELF_RELOC(R_SPARC_M44,          51)
ELF_RELOC(R_SPARC_L44,          52)
ELF_RELOC(R_SPARC_REGISTER,     53)
ELF_RELOC(R_SPARC_UA64,         54)
ELF_RELOC(R_SPARC_UA16,         55)
ELF_RELOC(R_SPARC_TLS_GD_HI22,    56)
ELF_RELOC(R_SPARC_TLS_GD_LO10,    57)
ELF_RELOC(R_SPARC_TLS_GD_ADD,     58)
ELF_RELOC(R_SPARC_TLS_GD_CALL,    59)
ELF_RELOC(R_SPARC_TLS_LDM_HI22,   60)
ELF_RELOC(R_SPARC_TLS_LDM_LO10,   61)
ELF_RELOC(R_SPARC_TLS_LDM_ADD,    62)
ELF_RELOC(R_SPARC_TLS_LDM_CALL,   63)
ELF_RELOC(R_SPARC_TLS_LDO_HIX22,  64)
ELF_RELOC(R_SPARC_TLS_LDO_LOX10,  65)
ELF_RELOC(R_SPARC_TLS_LDO_ADD,    66)
ELF_RELOC(R_SPARC_TLS_IE_HI22,    67)
ELF_RELOC(R_SPARC_TLS_IE_LO10,    68)
ELF_RELOC(R_SPARC_TLS_IE_LD,      69)
ELF_RELOC(R_SPARC_TLS_IE_LDX,     70)
ELF_RELOC(R_SPARC_TLS_IE_ADD,     71)
ELF_RELOC(R_SPARC_TLS_LE_HIX22,   72)
ELF_RELOC(R_SPARC_TLS_LE_LOX10,   73)
ELF_RELOC(R_SPARC_TLS_DTPMOD32,   74)
ELF_RELOC(R_SPARC_TLS_DTPMOD64,   75)
ELF_RELOC(R_SPARC_TLS_DTPOFF32,   76)
ELF_RELOC(R_SPARC_TLS_DTPOFF64,   77)
ELF_RELOC(R_SPARC_TLS_TPOFF32,    78)
ELF_RELOC(R_SPARC_TLS_TPOFF64,    79)
ELF_RELOC(R_SPARC_GOTDATA_HIX22,  80)
ELF_RELOC(R_SPARC_GOTDATA_LOX10,  81)
ELF_RELOC(R_SPARC_GOTDATA_OP_HIX22,  82)
ELF_RELOC(R_SPARC_GOTDATA_OP_LOX10,  83)
ELF_RELOC(R_SPARC_GOTDATA_OP,     84)
