
#ifndef ELF_RELOC
#error "ELF_RELOC must be defined"
#endif

ELF_RELOC(R_390_NONE,          0)
ELF_RELOC(R_390_8,             1)
ELF_RELOC(R_390_12,            2)
ELF_RELOC(R_390_16,            3)
ELF_RELOC(R_390_32,            4)
ELF_RELOC(R_390_PC32,          5)
ELF_RELOC(R_390_GOT12,         6)
ELF_RELOC(R_390_GOT32,         7)
ELF_RELOC(R_390_PLT32,         8)
ELF_RELOC(R_390_COPY,          9)
ELF_RELOC(R_390_GLOB_DAT,     10)
ELF_RELOC(R_390_JMP_SLOT,     11)
ELF_RELOC(R_390_RELATIVE,     12)
ELF_RELOC(R_390_GOTOFF,       13)
ELF_RELOC(R_390_GOTPC,        14)
ELF_RELOC(R_390_GOT16,        15)
ELF_RELOC(R_390_PC16,         16)
ELF_RELOC(R_390_PC16DBL,      17)
ELF_RELOC(R_390_PLT16DBL,     18)
ELF_RELOC(R_390_PC32DBL,      19)
ELF_RELOC(R_390_PLT32DBL,     20)
ELF_RELOC(R_390_GOTPCDBL,     21)
ELF_RELOC(R_390_64,           22)
ELF_RELOC(R_390_PC64,         23)
ELF_RELOC(R_390_GOT64,        24)
ELF_RELOC(R_390_PLT64,        25)
ELF_RELOC(R_390_GOTENT,       26)
ELF_RELOC(R_390_GOTOFF16,     27)
ELF_RELOC(R_390_GOTOFF64,     28)
ELF_RELOC(R_390_GOTPLT12,     29)
ELF_RELOC(R_390_GOTPLT16,     30)
ELF_RELOC(R_390_GOTPLT32,     31)
ELF_RELOC(R_390_GOTPLT64,     32)
ELF_RELOC(R_390_GOTPLTENT,    33)
ELF_RELOC(R_390_PLTOFF16,     34)
ELF_RELOC(R_390_PLTOFF32,     35)
ELF_RELOC(R_390_PLTOFF64,     36)
ELF_RELOC(R_390_TLS_LOAD,     37)
ELF_RELOC(R_390_TLS_GDCALL,   38)
ELF_RELOC(R_390_TLS_LDCALL,   39)
ELF_RELOC(R_390_TLS_GD32,     40)
ELF_RELOC(R_390_TLS_GD64,     41)
ELF_RELOC(R_390_TLS_GOTIE12,  42)
ELF_RELOC(R_390_TLS_GOTIE32,  43)
ELF_RELOC(R_390_TLS_GOTIE64,  44)
ELF_RELOC(R_390_TLS_LDM32,    45)
ELF_RELOC(R_390_TLS_LDM64,    46)
ELF_RELOC(R_390_TLS_IE32,     47)
ELF_RELOC(R_390_TLS_IE64,     48)
ELF_RELOC(R_390_TLS_IEENT,    49)
ELF_RELOC(R_390_TLS_LE32,     50)
ELF_RELOC(R_390_TLS_LE64,     51)
ELF_RELOC(R_390_TLS_LDO32,    52)
ELF_RELOC(R_390_TLS_LDO64,    53)
ELF_RELOC(R_390_TLS_DTPMOD,   54)
ELF_RELOC(R_390_TLS_DTPOFF,   55)
ELF_RELOC(R_390_TLS_TPOFF,    56)
ELF_RELOC(R_390_20,           57)
ELF_RELOC(R_390_GOT20,        58)
ELF_RELOC(R_390_GOTPLT20,     59)
ELF_RELOC(R_390_TLS_GOTIE20,  60)
ELF_RELOC(R_390_IRELATIVE,    61)
